// DesignView.cpp : implementation of the CDesignView class
//

#include "stdafx.h"
#include "Design.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDesignView

IMPLEMENT_DYNCREATE(CDesignView, CScrollView)

BEGIN_MESSAGE_MAP(CDesignView, CScrollView)
	//{{AFX_MSG_MAP(CDesignView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DELETE, OnUpdateEditDelete)

	ON_UPDATE_COMMAND_UI(ID_INDICATOR_WIDTH_HEIGHT, OnUpdateWidthHeight)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDesignView construction/destruction
void CDesignView::OnUpdateWidthHeight(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(m_Dragging); 
    CString strPage;
	CDesignDoc* pDoc = GetDocument();
	UINT currentelem = pDoc->m_CurrentElement;	// get current item
	if ((currentelem == ID_ELEMENT_ROOM) || (currentelem == ID_ELEMENT_ROOF))
	{
		strPage.Format( "x = %d, y = %d; Width = %d; Height = %d", m_PointOrigin.x/8, m_PointOrigin.y/8,
		abs(m_CurPos.x - m_PointOrigin.x/8) , abs(m_CurPos.y - m_PointOrigin.y/8));
	}
	else
	{
		strPage.Format( "x = %d, y = %d", m_CurPos.x, m_CurPos.y);
	}
    pCmdUI->SetText( strPage );
}

CDesignView::CDesignView()
{
	// TODO: add construction code here
//	pDoc->m_ElemSelected = -1;
	m_Deletable = TRUE;
	m_Dragging = 0;
//	pDoc->m_sel_pt_a = (0,0);
//	pDoc->m_sel_pt_b = (0,0);
	m_PenDotted.CreatePen(PS_DOT, 1, RGB(0,0,0));
	m_HArrow = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
	m_HCross = AfxGetApp()->LoadStandardCursor(IDC_CROSS);
}

CDesignView::~CDesignView()
{
}

BOOL CDesignView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs	
	m_Classname = AfxRegisterWndClass
		(CS_HREDRAW	|	CS_VREDRAW,
		0,
		(HBRUSH)::GetStockObject(WHITE_BRUSH),
		0);
	cs.lpszClass = m_Classname;

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDesignView drawing
void CDesignView::OnDraw(CDC* pDC)
{
	CDesignDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// TODO: add draw code for native data here
	CSize ScrollSize = GetTotalSize();
	CPen Pen, *pOldPen;
	
	//select pen
	Pen.CreatePen(PS_INSIDEFRAME, 1, RGB(192,192,192));		//grey
	pOldPen = pDC->SelectObject(&Pen);


	CRect	ClipRect;
	CRect	DimRect;
	CRect	IntRect;
	CElement *pElement;
	
	//Draw grid
	for (int i = 0; i <= ScrollSize.cx; i = i + 8)
	{
		pDC->MoveTo(i, 0);
		pDC->LineTo(i, ScrollSize.cy);
	}
	for (i = 0; i <= ScrollSize.cy; i = i + 8)
	{
		pDC->MoveTo(0, i);
		pDC->LineTo(ScrollSize.cx, i);
	}

	pDC->GetClipBox(&ClipRect);
	int NumElem = pDoc->GetNumElem();
	for (i = 0; i < NumElem; ++i)
	{
		pElement = pDoc->GetElem(i);
		if (pElement->m_ElemType == ROOF)
		{
			DimRect = pElement->GetDimRect();
			if(IntRect.IntersectRect(DimRect, ClipRect))
				pElement->Draw(pDC);
		}
	}

	
	for (i = 0; i < NumElem; ++i)
	{
		pElement = pDoc->GetElem(i);
		if (pElement->m_ElemType == ROOM)
		{
			DimRect = pElement->GetDimRect();
			if(IntRect.IntersectRect(DimRect, ClipRect))
				pElement->Draw(pDC);
		}
	}
	for (i = 0; i < NumElem; ++i)
	{
		pElement = pDoc->GetElem(i);
		if (pElement->m_ElemType == DOOR)
		{
			DimRect = pElement->GetDimRect();
			if(IntRect.IntersectRect(DimRect, ClipRect))
				pElement->Draw(pDC);
		}
	}
	for (i = 0; i < NumElem; ++i)
	{
		pElement = pDoc->GetElem(i);
		if (pElement->m_ElemType == WINDOW)
		{
			DimRect = pElement->GetDimRect();
			if(IntRect.IntersectRect(DimRect, ClipRect))
				pElement->Draw(pDC);
		}
	}

////Draw dotted rect here////
	pDC->SelectObject(&m_PenDotted);
	pDC->Rectangle(pDoc->m_sel_pt_a.x, pDoc->m_sel_pt_a.y, pDoc->m_sel_pt_b.x + 1, pDoc->m_sel_pt_b.y + 1);
	pDC->SelectObject(pOldPen);
}

/////////////////////////////////////////////////////////////////////////////
// CDesignView printing

BOOL CDesignView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDesignView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDesignView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CDesignView diagnostics

#ifdef _DEBUG
void CDesignView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CDesignView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CDesignDoc* CDesignView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDesignDoc)));
	return (CDesignDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDesignView message handlers

void CDesignView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	
	// TODO: Add your specialized code here and/or call the base class
	SetScrollSizes( MM_TEXT, GetDocument()->GetDocSize());
}

void CDesignView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CClientDC ClientDC(this);
	OnPrepareDC(&ClientDC);
	ClientDC.DPtoLP(&point);

	CDesignDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	UINT currentelem = pDoc->m_CurrentElement;	// get current item
	int x = point.x;
	int y = point.y;
	int NumElem, i, j;
	//////Snap to grid
	x = (point.x/8)*8;
	y = (point.y/8)*8;
	CPoint npoint(x, y);

////// Check if cursor is within drawing area
	CSize ScrollSize = GetTotalSize();
	CRect ScrollRect(0,0,ScrollSize.cx, ScrollSize.cy);
	if (!ScrollRect.PtInRect(npoint))
		return;

	CElement *pElement;
	CRect	Temprect, Temprect2;

////// Check if starting position is within drawable area (outside all rooms; for room selection only)
	if (currentelem == ID_ELEMENT_ROOM)
	{
		NumElem = pDoc->GetNumElem();
		for (i = 0; i < NumElem; ++i)
		{
			pElement = pDoc->GetElem(i);
			if (pElement->m_ElemType == ROOM)	// if it is a room
			{
				Temprect = pElement->GetDimRect();
				Temprect.DeflateRect(1,1);
				if (Temprect.PtInRect(npoint))	// starting point is located inside an existing room
					return;
			}
		}
	}

////// Modify and save cursor postion
	m_PointOrigin = npoint;

	ClientDC.SetROP2(R2_NOT);
	ClientDC.SelectObject(&m_PenDotted);
	ClientDC.SetBkMode(TRANSPARENT);
	ClientDC.SelectStockObject(NULL_BRUSH);
////// Draw dotted box if Door or Window is selected
	if ((currentelem == ID_ELEMENT_DOOR) || (currentelem == ID_ELEMENT_WINDOW))
	{
		if (m_PointOrigin.x < 24)
			m_PointOrigin.x = 24;
		if (m_PointOrigin.y < 24)
			m_PointOrigin.y = 24;
		ClientDC.Rectangle(m_PointOrigin.x - 24, m_PointOrigin.y - 24, m_PointOrigin.x, m_PointOrigin.y);
	}
	m_PointOld = m_PointOrigin;
	SetCapture();
	m_Dragging = 1;

///////	Select element to be deleted
	BOOL IsSelected = FALSE;
	if (currentelem == ID_ELEMENT_CURSOR)
	{
//		ClientDC.Rectangle(pDoc->m_sel_pt_a.x, pDoc->m_sel_pt_a.y, pDoc->m_sel_pt_b.x + 1, pDoc->m_sel_pt_b.y + 1);
///////	Search if any element is selected
		NumElem = pDoc->GetNumElem();
///////	Search door & window
		for (i = 0; ((i < NumElem) && (IsSelected == FALSE)); ++i)
		{
			pElement = pDoc->GetElem(i);
			if ((pElement->m_ElemType == DOOR) || (pElement->m_ElemType == WINDOW))	// if it is a door or window
			{
				Temprect = pElement->GetDimRect();
				if (Temprect.PtInRect(npoint))	// starting point is located inside an existing room
				{
					IsSelected = TRUE;
					pDoc->m_ElemSelected = i;
					m_Deletable = TRUE;
					pDoc->m_sel_pt_a.x = Temprect.TopLeft().x;
					pDoc->m_sel_pt_a.y = Temprect.TopLeft().y;
					pDoc->m_sel_pt_b.x = Temprect.BottomRight().x;
					pDoc->m_sel_pt_b.y = Temprect.BottomRight().y;
				}
			}
		}
///////	Search roof
		for (i = 0; ((i < NumElem) && (IsSelected == FALSE)); ++i)
		{
			pElement = pDoc->GetElem(i);
			if (pElement->m_ElemType == ROOF)	// if it is a roof
			{
				Temprect = pElement->GetDimRect();
				if (Temprect.PtInRect(npoint))	// starting point is located inside an existing room
				{
					IsSelected = TRUE;
					pDoc->m_ElemSelected = i;
					m_Deletable = TRUE;
					pDoc->m_sel_pt_a.x = Temprect.TopLeft().x;
					pDoc->m_sel_pt_a.y = Temprect.TopLeft().y;
					pDoc->m_sel_pt_b.x = Temprect.BottomRight().x;
					pDoc->m_sel_pt_b.y = Temprect.BottomRight().y;
				}
			}
		}
/////// Search room (if found, check if any door, window, or roof is associated to it)
		for (i = 0; ((i < NumElem) && (IsSelected == FALSE)); ++i)
		{
			pElement = pDoc->GetElem(i);
			if (pElement->m_ElemType == ROOM)	// if it is a door or window
			{
				CRect IntRect;

//				if (IntRect.IntersectRect(Temprect, Currentrect))
				Temprect = pElement->GetDimRect();
				if (Temprect.PtInRect(npoint))		// starting point is located inside an existing room
				{
					IsSelected = TRUE;
					pDoc->m_ElemSelected = i;
					m_Deletable = TRUE;
					pDoc->m_sel_pt_a.x = Temprect.TopLeft().x;
					pDoc->m_sel_pt_a.y = Temprect.TopLeft().y;
					pDoc->m_sel_pt_b.x = Temprect.BottomRight().x;
					pDoc->m_sel_pt_b.y = Temprect.BottomRight().y;
///////	Check if it has any other components attached to it
					CElement *pSearchElem;
					for (j = 0; ((j < NumElem) && (m_Deletable == TRUE)); ++j)
					{
						pSearchElem = pDoc->GetElem(j);
						if (pSearchElem->m_ElemType != ROOM)
						{
							Temprect2 = pSearchElem->GetDimRect();
							if (IntRect.IntersectRect(Temprect, Temprect2))
							{
								m_Deletable = FALSE;
							}
						}
					}
				}
			}
		}

		if (IsSelected == FALSE)
		{
			pDoc->m_ElemSelected = -1;
			pDoc->m_sel_pt_a = (0,0);
			pDoc->m_sel_pt_b = (0,0);
		}
		Invalidate(FALSE);
	}
///////	Clip mouse cursor:
	ClientDC.LPtoDP(&ScrollRect);
	CRect ViewRect;
	GetClientRect(&ViewRect);
	CRect IntRect;
	IntRect.IntersectRect(&ScrollRect, &ViewRect);
	ClientToScreen(&IntRect);
	::ClipCursor(&IntRect);

	CScrollView::OnLButtonDown(nFlags, point);
}

void CDesignView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CDesignDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!m_Dragging)
		return;

	m_Dragging = 0;
	::ReleaseCapture();
	::ClipCursor(NULL);
	CClientDC ClientDC(this);
	OnPrepareDC(&ClientDC);
	ClientDC.DPtoLP(&point);
	ClientDC.SetROP2(R2_NOT);
	ClientDC.SelectObject(&m_PenDotted);
	ClientDC.SetBkMode(TRANSPARENT);
	ClientDC.SelectStockObject(NULL_BRUSH);

////// delete previously drawn rectangle (for Door or Window)
	switch (pDoc->m_CurrentElement)
	{
	case ID_ELEMENT_DOOR:
		ClientDC.Rectangle(m_PointOld.x - 24, m_PointOld.y - 24, m_PointOld.x, m_PointOld.y);
		break;
	case ID_ELEMENT_WINDOW:
		ClientDC.Rectangle(m_PointOld.x - 24, m_PointOld.y - 24, m_PointOld.x, m_PointOld.y);
		break;
	}

////// Snap to grid
	int x = point.x;
	int y = point.y;
	x = (point.x/8)*8;
	y = (point.y/8)*8;
	CPoint npoint(x, y);

////// Do nothing in these cases
	if (pDoc->m_CurrentElement == ID_ELEMENT_CURSOR)	//if cursor is selected
		return;
	if ((m_PointOld == m_PointOrigin) && (pDoc->m_CurrentElement == ID_ELEMENT_ROOM))		// if two points are the same
		return;
	CElement *pElement;

	int NumElem = pDoc->GetNumElem();
	int i;
	CRect Temprect;

//////	create new element
	switch (pDoc->m_CurrentElement)
	{
//////	create new room
	case ID_ELEMENT_ROOM:
		{
//////////	Prevent drawing into existing room; only for ROOM
			CRect Currentrect(	min(m_PointOrigin.x, x),
								min(m_PointOrigin.y, y),
								max(m_PointOrigin.x, x),
								max(m_PointOrigin.y, y));
			ASSERT_VALID(pDoc);
			for (i = 0; i < NumElem; ++i)		// check existing rooms
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOM)	// if it is a room
				{
					Temprect = pElement->GetDimRect();
					Temprect.DeflateRect(1,1);
					CRect IntRect;
					if (IntRect.IntersectRect(Temprect, Currentrect))
					{
						npoint = m_PointOld;
					}
				}
			}
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
			pElement = new CRoom(m_PointOrigin.x, m_PointOrigin.y, npoint.x, npoint.y, pDoc->m_Color, pDoc->m_Thickness);
		}
		break;
	case ID_ELEMENT_DOOR:
		{
//////	create new door
			BOOL intersectdoor = FALSE;
			CPoint pointa, pointb;
////////////// Check if there is intersection between wall and door
			for (i = 0; ((i < NumElem) && (intersectdoor == FALSE)); ++i)
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOM)
				{
					Temprect = pElement->GetDimRect();
					int top = (Temprect.TopLeft()).y;
					int left = (Temprect.TopLeft()).x;
					int bottom = (Temprect.BottomRight()).y;
					int right = (Temprect.BottomRight()).x;

					if (npoint.x < 24)
						npoint.x = 24;
					if (npoint.y < 24)
						npoint.y = 24;
					if ((npoint.x > left) && (npoint.x - 24 < left) && (npoint.y < bottom) && (npoint.y > top + 24))
					{
						pointa.x = left - 8;
						pointb.x = left + 8;
						pointa.y = npoint.y - 24;
						pointb.y = npoint.y;
						intersectdoor = TRUE;
					}
					else if ((npoint.x > right) && (npoint.x - 24 < right) && (npoint.y < bottom) && (npoint.y > top + 24))
					{
						pointa.x = right - 8;
						pointb.x = right + 8;
						pointa.y = npoint.y - 24;
						pointb.y = npoint.y;
						intersectdoor = TRUE;
					}
					else if ((npoint.y > bottom) && (npoint.y - 24 < bottom) && (npoint.x < right) && (npoint.x > left + 24))
					{
						pointa.x = npoint.x -24;
						pointb.x = npoint.x;
						pointa.y = bottom - 8;
						pointb.y = bottom + 8;
						intersectdoor = TRUE;
					}
					else if ((npoint.y > top) && (npoint.y - 24 < top) && (npoint.x < right) && (npoint.x > left + 24))
					{
						pointa.x = npoint.x -24;
						pointb.x = npoint.x;
						pointa.y = top - 8;
						pointb.y = top + 8;
						intersectdoor = TRUE;
					}
				}
			}
			if (intersectdoor == FALSE)
				return;
			CRect intersect;
			CRect Temprect2(pointa.x, pointa.y, pointb.x, pointb.y);
			for (i = 0; i < NumElem; ++i)
			{
				pElement = pDoc->GetElem(i);
				if ((pElement->m_ElemType == DOOR) || (pElement->m_ElemType == WINDOW))
				{
					Temprect = pElement->GetDimRect();
					intersectdoor = !intersect.IntersectRect(Temprect, Temprect2);
					if (intersectdoor == FALSE)
						return;
				}
			}
			pElement = new CDoor(pointa.x, pointa.y, pointb.x, pointb.y, pDoc->m_Color, pDoc->m_Thickness);
		}
		break;
	case ID_ELEMENT_WINDOW:
		{
//			ClientDC.Rectangle(m_PointOld.x - 24, m_PointOld.y - 24, m_PointOld.x, m_PointOld.y);
			BOOL intersectwindow = FALSE;
			CPoint pointa, pointb;
			int outxy;
////////////// Check if there is intersection between wall and door
			for (i = 0; ((i < NumElem) && (intersectwindow == FALSE)); ++i)
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOM)
				{
					Temprect = pElement->GetDimRect();
					int top = (Temprect.TopLeft()).y;
					int left = (Temprect.TopLeft()).x;
					int bottom = (Temprect.BottomRight()).y;
					int right = (Temprect.BottomRight()).x;

					if (npoint.x < 24)
						npoint.x = 24;
					if (npoint.y < 24)
						npoint.y = 24;
					if ((npoint.x > left) && (npoint.x - 24 < left) && (npoint.y < bottom) && (npoint.y > top + 24))
					{
						pointa.x = left - 8;
						pointb.x = left + 8;
						pointa.y = npoint.y - 24;
						pointb.y = npoint.y;
						outxy = OPENLEFT;
						intersectwindow = TRUE;
					}
					else if ((npoint.x > right) && (npoint.x - 24 < right) && (npoint.y < bottom) && (npoint.y > top + 24))
					{
						pointa.x = right - 8;
						pointb.x = right + 8;
						pointa.y = npoint.y - 24;
						pointb.y = npoint.y;
						outxy = OPENRIGHT;
						intersectwindow = TRUE;
					}
					else if ((npoint.y > bottom) && (npoint.y - 24 < bottom) && (npoint.x < right) && (npoint.x > left + 24))
					{
						pointa.x = npoint.x -24;
						pointb.x = npoint.x;
						pointa.y = bottom - 8;
						pointb.y = bottom + 8;
						outxy = OPENDOWN;
						intersectwindow = TRUE;
					}
					else if ((npoint.y > top) && (npoint.y - 24 < top) && (npoint.x < right) && (npoint.x > left + 24))
					{
						pointa.x = npoint.x -24;
						pointb.x = npoint.x;
						pointa.y = top - 8;
						pointb.y = top + 8;
						outxy = OPENUP;
						intersectwindow = TRUE;
					}
				}
			}
			if (intersectwindow == FALSE)
				return;
			CRect intersect;
			CRect Temprect2(pointa.x, pointa.y, pointb.x, pointb.y);
			for (i = 0; i < NumElem; ++i)
			{
				pElement = pDoc->GetElem(i);
				if ((pElement->m_ElemType == DOOR) || (pElement->m_ElemType == WINDOW))
				{
					Temprect = pElement->GetDimRect();
					intersectwindow = !intersect.IntersectRect(Temprect, Temprect2);
					if (intersectwindow == FALSE)
						return;
				}
			}
			pElement = new CWindow(pointa.x, pointa.y, pointb.x, pointb.y, outxy, pDoc->m_Color, pDoc->m_Thickness);
		}
		break;
	case ID_ELEMENT_ROOF:
		{
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
//////////	Prevent drawing outside a room
			BOOL intersect = FALSE;
			CRect Currentrect(	min(m_PointOrigin.x, x),
								min(m_PointOrigin.y, y),
								max(m_PointOrigin.x, x),
								max(m_PointOrigin.y, y));
			ASSERT_VALID(pDoc);
			for (i = 0; i < NumElem; ++i)		// check existing roofs
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOF)	// if it is a roof, use Old Point
				{
					Temprect = pElement->GetDimRect();
					Temprect.DeflateRect(1,1);
					CRect IntRect;
					if (IntRect.IntersectRect(Temprect, Currentrect))
					{
						npoint = m_PointOld;
					}
				}
			}
			Currentrect.SetRect(	min(m_PointOrigin.x, npoint.x),
									min(m_PointOrigin.y, npoint.y),
									max(m_PointOrigin.x, npoint.x),
									max(m_PointOrigin.y, npoint.y));
			for (i = 0; ((i < NumElem) && (intersect == FALSE)); ++i)		// check existing rooms
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOM)	// if it is a room, return
				{
					Temprect = pElement->GetDimRect();
					Temprect.DeflateRect(1,1);
					CRect IntRect;
					if (IntRect.IntersectRect(Temprect, Currentrect))
					{
						intersect = TRUE;
					}
				}
			}			
			if (intersect == FALSE)
				return;

			//////////	Choose which side it should slope towards
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
			CSlopeDlg dlg;
			if (dlg.DoModal() == IDOK)
			{
				ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
				pElement = new CRoof(m_PointOrigin.x, m_PointOrigin.y, npoint.x, npoint.y, pDoc->m_Color, pDoc->m_Thickness, dlg.m_type);
			}
			else
			{
///////////////	Cancel button is pressed
				ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
				return;
			}
		}
		break;
	}
	
	ClientDC.SetROP2(R2_COPYPEN);

	CDesignDoc* PDoc = GetDocument();
	PDoc->AddElem(pElement);
	pDoc->UpdateAllViews(NULL, 0, pElement);

	CScrollView::OnLButtonUp(nFlags, point);
}

void CDesignView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CClientDC ClientDC(this);
	OnPrepareDC (&ClientDC);
	ClientDC.DPtoLP(&point);
	m_CurPos.x = point.x/8;
	m_CurPos.y = point.y/8;
	CDesignDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	UINT currentelem = pDoc->m_CurrentElement;

	if (!m_Dragging)
	{
		CSize ScrollSize = GetTotalSize();
		CRect ScrollRect(0,0, ScrollSize.cx, ScrollSize.cy);
		if ((!ScrollRect.PtInRect(point)) || (currentelem == ID_ELEMENT_CURSOR))
			::SetCursor(m_HArrow);
		else
			::SetCursor(m_HCross);
		return;
	}

	ClientDC.SetROP2(R2_NOT);
	ClientDC.SelectObject(&m_PenDotted);
	ClientDC.SetBkMode(TRANSPARENT);
	ClientDC.SelectStockObject(NULL_BRUSH);

	int x = point.x;
	int y = point.y;
//////Snap to grid
	x = (point.x/8)*8;
	y = (point.y/8)*8;
	CPoint npoint(x, y);
	CRect Currentrect(	min(m_PointOrigin.x, x),
						min(m_PointOrigin.y, y),
						max(m_PointOrigin.x, x),
						max(m_PointOrigin.y, y));

	ASSERT_VALID(pDoc);
	CElement *pElement;
	CRect	Temprect;

////// Same as IsRectNull--check if the current rectangle's null
	if (((npoint.x == m_PointOrigin.x) || (npoint.y == m_PointOrigin.y)) && (currentelem == ID_ELEMENT_ROOM))
		return;

	if ((currentelem == ID_ELEMENT_DOOR) || (currentelem == ID_ELEMENT_WINDOW))
	{
		if (npoint.x < 24)
			npoint.x = 24;
		if (npoint.y < 24)
			npoint.y = 24;
	}

//////	Select element
	switch(currentelem)
	{
	case ID_ELEMENT_ROOM:
		{
////////// prevent drawing into existing room (only for ROOM element)
			int NumElem = pDoc->GetNumElem();
			for (int i = 0; i < NumElem; ++i)
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOM)	// if it is a room
				{
					Temprect = pElement->GetDimRect();
					Temprect.DeflateRect(1,1);
					CRect IntRect;
					if (IntRect.IntersectRect(Temprect, Currentrect))
					{
						return;
					}
				}
			}
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, npoint.x, npoint.y);
		}
		break;
	case ID_ELEMENT_DOOR:
		ClientDC.Rectangle(m_PointOld.x - 24, m_PointOld.y - 24, m_PointOld.x, m_PointOld.y);
		ClientDC.Rectangle(npoint.x - 24, npoint.y - 24, npoint.x, npoint.y);
		break;
	case ID_ELEMENT_WINDOW:
		ClientDC.Rectangle(m_PointOld.x - 24, m_PointOld.y - 24, m_PointOld.x, m_PointOld.y);
		ClientDC.Rectangle(npoint.x - 24, npoint.y - 24, npoint.x, npoint.y);
		break;
	case ID_ELEMENT_ROOF:
		{
			int NumElem = pDoc->GetNumElem();
			for (int i = 0; i < NumElem; ++i)
			{
				pElement = pDoc->GetElem(i);
				if (pElement->m_ElemType == ROOF)	// if it is a room
				{
					Temprect = pElement->GetDimRect();
					Temprect.DeflateRect(1,1);
					CRect IntRect;
					if (IntRect.IntersectRect(Temprect, Currentrect))
					{
						return;
					}
				}
			}
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, m_PointOld.x, m_PointOld.y);
			ClientDC.Rectangle(m_PointOrigin.x, m_PointOrigin.y, npoint.x, npoint.y);
		}
		break;
	}

	m_PointOld = npoint;
	CScrollView::OnMouseMove(nFlags, point);
}

void CDesignView::OnEditDelete() 
{
	// TODO: Add your command handler code here
	CDesignDoc* pDoc = GetDocument();
	if (pDoc->m_ElemSelected == -1)
		return;
	// Delete currently selected element
	if (m_Deletable == FALSE)
	{
		AfxMessageBox("Please delete associated components first.");
		return;
	}
	int Index = pDoc->m_ElemArray.GetUpperBound();
	if (Index > -1)
	{
		delete pDoc->m_ElemArray.GetAt(pDoc->m_ElemSelected);
		pDoc->m_ElemArray.RemoveAt(pDoc->m_ElemSelected);
	}
	pDoc->UpdateAllViews(0);
	pDoc->SetModifiedFlag();
	pDoc->m_sel_pt_a = (0,0);
	pDoc->m_sel_pt_b = (0,0);
	pDoc->m_ElemSelected = -1;
}

void CDesignView::OnUpdateEditDelete(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	CDesignDoc* pDoc = GetDocument();
	pCmdUI->Enable(pDoc->m_ElemSelected != -1);
}





/////////////////////////////////////////////////////////////////////////////
// CModelView

IMPLEMENT_DYNCREATE(CModelView, CView)

CModelView::CModelView()
{
	m_hGLContext = NULL;
	m_GLPixelIndex = 0;
	m_LButtonDown = FALSE;
	m_RButtonDown = FALSE;
	m_BuildList = FALSE;
}

CModelView::~CModelView()
{
}


BEGIN_MESSAGE_MAP(CModelView, CView)
	//{{AFX_MSG_MAP(CModelView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CModelView drawing

void CModelView::OnDraw(CDC* pDC)
{
	CDesignDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CModelView diagnostics

#ifdef _DEBUG
void CModelView::AssertValid() const
{
	CView::AssertValid();
}

void CModelView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CDesignDoc* CModelView::GetDocument()
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDesignDoc)));
	return (CDesignDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CModelView message handlers

BOOL CModelView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
	return CView::PreCreateWindow(cs);
}

BOOL CModelView::SetWindowPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pixelDesc;

	pixelDesc.nSize			= sizeof (PIXELFORMATDESCRIPTOR);
	pixelDesc.nVersion		= 1;
	pixelDesc.dwFlags		=	PFD_DRAW_TO_WINDOW	|
								PFD_SUPPORT_OPENGL	|
								PFD_DOUBLEBUFFER	|
								PFD_STEREO_DONTCARE;
	pixelDesc.iPixelType	= PFD_TYPE_RGBA;
	pixelDesc.cColorBits	= 32;
	pixelDesc.cRedBits		= 8;
	pixelDesc.cRedShift		= 16;
	pixelDesc.cGreenBits	= 8;
	pixelDesc.cGreenShift	= 8;
	pixelDesc.cBlueBits		= 8;
	pixelDesc.cBlueShift	= 0;
	pixelDesc.cAlphaBits	= 0;
	pixelDesc.cAlphaShift	= 0;
	pixelDesc.cAccumBits	= 64;
	pixelDesc.cAccumRedBits	= 16;
	pixelDesc.cAccumGreenBits	= 16;
	pixelDesc.cAccumBlueBits	= 16;
	pixelDesc.cAccumAlphaBits	= 0;
	pixelDesc.cDepthBits	= 32;
	pixelDesc.cStencilBits	= 8;
	pixelDesc.cAuxBuffers	= 0;
	pixelDesc.cAuxBuffers	= 0;
	pixelDesc.iLayerType	= PFD_MAIN_PLANE;
	pixelDesc.bReserved		= 0;
	pixelDesc.dwLayerMask	= 0;
	pixelDesc.dwVisibleMask	= 0;
	pixelDesc.dwDamageMask	= 0;

	m_GLPixelIndex = ChoosePixelFormat(hDC, &pixelDesc);
	if (m_GLPixelIndex ==0)	//Let's choose a default index.
	{
		m_GLPixelIndex = 1;
		if (DescribePixelFormat(hDC, m_GLPixelIndex, sizeof(PIXELFORMATDESCRIPTOR), &pixelDesc)==0)
		{
			return FALSE;
		}
	}
	if (SetPixelFormat( hDC, m_GLPixelIndex, &pixelDesc)==FALSE)
	{
		return FALSE;
	}
	return TRUE;
}

int CModelView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	HWND hWnd = GetSafeHwnd();
	HDC hDC = ::GetDC(hWnd);

	if (SetWindowPixelFormat(hDC) == FALSE)
		return 0;

	if (CreateViewGLContext(hDC) == FALSE)
		return 0;

	return 0;
}

BOOL CModelView::CreateViewGLContext(HDC hDC)
{
	m_hGLContext = wglCreateContext(hDC);
	if (m_hGLContext == NULL)
	{
		return FALSE;
	}
	if (wglMakeCurrent(hDC, m_hGLContext) == FALSE)
	{
		return FALSE;
	}
	return TRUE;
}

void CModelView::OnDestroy() 
{
	if (wglGetCurrentContext()!=NULL)
	{
		// make the rendering context not current
		wglMakeCurrent(NULL, NULL);
	}
	if (m_hGLContext!=NULL)
	{
		wglDeleteContext(m_hGLContext);
		m_hGLContext = NULL;
	}

	// Now the associated DC can be released
	CView::OnDestroy();
}

void CModelView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLsizei	width, height;
	GLdouble aspect;

	width = cx;
	height = cy;

	if (cy == 0)
		aspect = (GLdouble)width;
	else
		aspect = (GLdouble)width/(GLdouble)height;

	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(30, aspect, 1, 1200);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDrawBuffer(GL_BACK);
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);

}

void CModelView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CDesignDoc* pDoc = GetDocument();

////add vertex list here//////////////////////
	if (m_BuildList == FALSE)
	{
		glNewList(DOOR, GL_COMPILE);
			glBegin(GL_QUADS);
				glNormal3d(  0.0,  0.0,  1.0);
				glVertex3d(  0.15/10.0,  0.0, 0.1/10.0);
				glVertex3d(  0.15/10.0,  56.0/10.0, 0.1/10.0);
				glVertex3d(  23.85/10.0, 56.0/10.0, 0.1/10.0);
				glVertex3d(  23.85/10.0, 0.0, 0.1/10.0);
			glEnd();
			glBegin(GL_QUADS);
				glNormal3d(  0.0,  0.0,  -1.0);
				glVertex3d(  0.15/10.0,  0.0, -0.1/10.0);
				glVertex3d(  0.15/10.0,  56.0/10.0, -0.1/10.0);
				glVertex3d(  23.85/10.0, 56.0/10.0, -0.1/10.0);
				glVertex3d(  23.85/10.0, 0.0, -0.1/10.0);
			glEnd();
		glEndList();

		glNewList(WINDOW, GL_COMPILE);
			glBegin(GL_QUADS);
				glNormal3d(  0.0,  0.0,  1.0);
				glVertex3d(  0.0,  0.0, 0.1/10.0);
				glVertex3d(  0.0,  32.0/10.0, 0.1/10.0);
				glVertex3d(  24/10.0, 32.0/10.0, 0.1/10.0);
				glVertex3d(  24/10.0, 0.0, 0.1/10.0);
			glEnd();
			glBegin(GL_QUADS);
				glNormal3d(  0.0,  0.0,  -1.0);
				glVertex3d(  0.0,  0.0, -0.1/10.0);
				glVertex3d(  0.0,  32.0/10.0, -0.1/10.0);
				glVertex3d(  24/10.0, 32.0/10.0, -0.1/10.0);
				glVertex3d(  24/10.0, 0.0, -0.1/10.0);
			glEnd();
		glEndList();

		m_BuildList = TRUE;
	}
//////////////////////////////////////////////
	pDoc->RenderScene();

	SwapBuffers(dc.m_ps.hdc);
	// Do not call CView::OnPaint() for painting messages
}

void CModelView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CDesignDoc* pDoc = GetDocument();
	if (pDoc->m_CurrentMode == ID_VIEW_NORMAL)
	{
		m_LButtonDown = TRUE;
		m_LDownPos = point;
		SetCapture();
		CRect ViewRect;
		GetWindowRect(&ViewRect);
		::ClipCursor(&ViewRect);
	}
	CView::OnLButtonDown(nFlags, point);
}

void CModelView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CDesignDoc* pDoc = GetDocument();
	if (pDoc->m_CurrentMode == ID_VIEW_NORMAL)
	{
		m_LButtonDown = FALSE;	
		::ReleaseCapture();
		::ClipCursor(NULL);
	}
	CView::OnLButtonUp(nFlags, point);
}

void CModelView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CDesignDoc* pDoc = GetDocument();
	if (m_LButtonDown)
	{
		CSize rotate = m_LDownPos - point;
		m_LDownPos = point;
		pDoc->m_yRotate -= rotate.cx/2;
		if ((pDoc->m_xRotate - rotate.cy/2 < 89)&&(pDoc->m_xRotate - rotate.cy/2 > -89))
			pDoc->m_xRotate -= rotate.cy/2;
		InvalidateRect(NULL, FALSE);
	}
	CView::OnMouseMove(nFlags, point);
}

void CModelView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	CDesignDoc* pDoc = GetDocument();

	if (pDoc->m_CurrentMode == ID_VIEW_WALKTHROUGH)
	{
		double diffx = (pDoc->eyex - pDoc->centerx)/2;
		double diffy = (pDoc->eyey - pDoc->centery)/2;
		double diffz = (pDoc->eyez - pDoc->centerz)/2;

		switch (nChar)
		{
		case '5':
			pDoc->eyex = pDoc->eyex - diffx;
			pDoc->eyey = pDoc->eyey - diffy;
			pDoc->eyez = pDoc->eyez - diffz;
			pDoc->centerx = pDoc->centerx - diffx;
			pDoc->centery = pDoc->centery - diffy;
			pDoc->centerz = pDoc->centerz - diffz;
			break;
		case '0':
			pDoc->eyex = pDoc->eyex + diffx;
			pDoc->eyey = pDoc->eyey + diffy;
			pDoc->eyez = pDoc->eyez + diffz;
			pDoc->centerx = pDoc->centerx + diffx;
			pDoc->centery = pDoc->centery + diffy;
			pDoc->centerz = pDoc->centerz + diffz;
			break;
		case '8':
			pDoc->m_yz_angle = pDoc->m_yz_angle + 0.02;

			pDoc->centerx = pDoc->eyex + cos(pDoc->m_yz_angle)*sin(pDoc->m_xz_angle);
			pDoc->centery = pDoc->eyey + sin(pDoc->m_yz_angle);
			pDoc->centerz = pDoc->eyez - cos(pDoc->m_yz_angle)*cos(pDoc->m_xz_angle);
			break;
		case '2':
			pDoc->m_yz_angle = pDoc->m_yz_angle - 0.02;
			
			pDoc->centerx = pDoc->eyex + cos(pDoc->m_yz_angle)*sin(pDoc->m_xz_angle);
			pDoc->centery = pDoc->eyey + sin(pDoc->m_yz_angle);
			pDoc->centerz = pDoc->eyez - cos(pDoc->m_yz_angle)*cos(pDoc->m_xz_angle);
			break;
		case '4':
			pDoc->m_xz_angle = pDoc->m_xz_angle - 0.02;

			pDoc->centerx = pDoc->eyex + cos(pDoc->m_yz_angle)*sin(pDoc->m_xz_angle);
			pDoc->centery = pDoc->eyey + sin(pDoc->m_yz_angle);
			pDoc->centerz = pDoc->eyez - cos(pDoc->m_yz_angle)*cos(pDoc->m_xz_angle);
			break;
		case '6':
			pDoc->m_xz_angle = pDoc->m_xz_angle + 0.02;

			pDoc->centerx = pDoc->eyex + cos(pDoc->m_yz_angle)*sin(pDoc->m_xz_angle);
			pDoc->centery = pDoc->eyey + sin(pDoc->m_yz_angle);
			pDoc->centerz = pDoc->eyez - cos(pDoc->m_yz_angle)*cos(pDoc->m_xz_angle);
			break;
		}
		InvalidateRect(NULL, FALSE);
	}
	CView::OnChar(nChar, nRepCnt, nFlags);
}

